#include "Pair.h"
#include <catch.h>
#include <sstream>
#include <string>

TEST_CASE("Construct")
{
    Pair<int, double> p(4, 5.5);

    CHECK(4 == p.first);
    CHECK(5.5 == p.second);
}

TEST_CASE("auto")
{
    auto p1 = Pair<bool, std::string>(true, "hello");
    auto p2 = make_Pair(true, std::string("hello"));
    CHECK( p1 == p2 );
}

TEST_CASE("stream insertion")
{
    std::ostringstream ss;
    ss << make_Pair(5, 6.5);
    CHECK( ss.str() == "(5, 6.5)" );
}

TEST_CASE("Compare")
{
    Pair<int, double> p1(4, 5.5);
    Pair<int, double> p2(4, 5.7);
    Pair<int, double> p3(5, 3);

    CHECK_FALSE(p1 < p1);
    CHECK(p1 < p2);
    CHECK(p1 < p3);
    CHECK_FALSE(p2 < p1);
    CHECK_FALSE(p2 < p2);
    CHECK(p2 < p3);
    CHECK_FALSE(p3 < p1);
    CHECK_FALSE(p3 < p2);
    CHECK_FALSE(p3 < p3);
}

TEST_CASE("Generic")
{
    Pair<std::string, char> p1("goodbye", 'd');
    Pair<std::string, char> p2("hello", 'b');
    Pair<std::string, char> p3("hello", 'c');

    CHECK_FALSE(p1 < p1);
    CHECK(p1 < p2);
    CHECK(p1 < p3);
    CHECK_FALSE(p2 < p1);
    CHECK_FALSE(p2 < p2);
    CHECK(p2 < p3);
    CHECK_FALSE(p3 < p1);
    CHECK_FALSE(p3 < p2);
    CHECK_FALSE(p3 < p3);
}

TEST_CASE("get")
{
    Pair<std::string, int> p("hello", 5);

    CHECK(get<0>(p) == "hello");
    CHECK(get<1>(p) == 5);

    get<int>(p) = 6;

    CHECK(get<1>(p) == 6);
}
