#include "String_vector.h"

#include <catch.h>

#include <iostream>

TEST_CASE("New_empty")
{
    String_vector v;
}

TEST_CASE("New_with_capacity")
{
    String_vector v(16);
}

TEST_CASE("Initializer_list")
{
    String_vector v{"1", "2", "3"};

    CHECK(3 == v.size());
    CHECK("1" == v[0]);
    CHECK("2" == v[1]);
    CHECK("3" == v[2]);
}

TEST_CASE("Copy_constructor")
{
    String_vector u{"1", "2", "3"};
    String_vector v(u);
    String_vector w(v);

    CHECK(u == w);

    v[0] = "5";
    CHECK(u == w);
    CHECK(u != v);
}

TEST_CASE("Assignment_operator")
{
    String_vector v{"1", "2", "3"};
    String_vector w;

    CHECK(v != w);
    w = v;
    CHECK(v == w);
    w.push_back("4");
    CHECK(v != w);
    w = v;
    CHECK(v == w);
}

TEST_CASE("Empty")
{
    String_vector v;
    CHECK(v.empty());
    v.push_back("0");
    CHECK_FALSE(v.empty());
}

TEST_CASE("Size")
{
    String_vector v;
    CHECK(0 == v.size());
    v.push_back("1");
    CHECK(1 == v.size());
    v.push_back("2");
    CHECK(2 == v.size());
    v.push_back("3");
    CHECK(3 == v.size());
    v.push_back("4");
    CHECK(4 == v.size());
}

TEST_CASE("Push_back")
{
    String_vector v;

    v.push_back("5");
    v.push_back("6");
    v.push_back("7");
    v.push_back("8");
    v.push_back("9");
    v.push_back("10");
    v.push_back("11");
    v.push_back("12");
    v.push_back("13");

    CHECK("5" == v[0]);
    CHECK("6" == v[1]);
    CHECK("7" == v[2]);
    CHECK("13" == v[8]);
}

TEST_CASE("Pop_back")
{
    String_vector v{"1", "2", "3"};
    CHECK(3 == v.size());
    v.pop_back();
    CHECK(2 == v.size());
    v.pop_back();
    CHECK(1 == v.size());
    v.pop_back();
    CHECK(0 == v.size());
}

TEST_CASE("Clear")
{
    String_vector v{"1", "2", "3"};
    CHECK(3 == v.size());
    v.clear();
    CHECK(0 == v.size());
    v.push_back("4");
    CHECK(1 == v.size());
}

TEST_CASE("Front_const")
{
    const String_vector v{"1", "2", "3"};
    CHECK("1" == v.front());
}

TEST_CASE("Front_assign")
{
    String_vector v{"1", "2", "3"};
    CHECK("1" == v.front());
    v.front() = "10";
    CHECK("10" == v.front());
}

TEST_CASE("Back_const")
{
    const String_vector v{"1", "2", "3"};
    CHECK("3" == v.back());
}

TEST_CASE("Back_assign")
{
    String_vector v{"1", "2", "3"};
    CHECK("3" == v.back());
    v.back() = "10";
    CHECK("10" == v.back());
}

TEST_CASE("Index_const")
{
    const String_vector v{"1", "2", "3"};
    CHECK("1" == v[0]);
    CHECK("2" == v[1]);
    CHECK("3" == v[2]);
}

TEST_CASE("Index_assign")
{
    String_vector v{"1", "2", "3"};
    CHECK("1" == v[0]);
    CHECK("2" == v[1]);
    CHECK("3" == v[2]);

    v[1] = "10";

    CHECK("1" == v[0]);
    CHECK("10" == v[1]);
    CHECK("3" == v[2]);
}

TEST_CASE("At_const")
{
    const String_vector v{"1", "2", "3"};
    CHECK("1" == v.at(0));
    CHECK("2" == v.at(1));
    CHECK("3" == v.at(2));
    CHECK_THROWS_AS(v.at(3), std::range_error);
}

TEST_CASE("At_assign")
{
    String_vector v{"1", "2", "3"};
    CHECK("1" == v.at(0));
    CHECK("2" == v.at(1));
    CHECK("3" == v.at(2));

    v.at(1) = "10";

    CHECK("1" == v.at(0));
    CHECK("10" == v.at(1));
    CHECK("3" == v.at(2));
}

TEST_CASE("Iteration")
{
    String_vector v;
    v.push_back("5");
    v.push_back("6");
    v.push_back("7");
    v.push_back("8");

    auto i = std::begin(v);
    auto j = std::end(v);
    CHECK(i != j);
    CHECK("5" == *i);

    ++i;
    CHECK(i != j);
    CHECK("6" == *i);

    ++i;
    CHECK(i != j);
    CHECK("7" == *i);

    ++i;
    CHECK(i != j);
    CHECK("8" == *i);

    ++i;
    CHECK(j == i);
}

TEST_CASE("Comparisons")
{
    String_vector u{"1", "2", "3"};
    String_vector v{"1", "2", "3", "4"};
    String_vector w{"1", "4", "3"};

    CHECK_FALSE(u < u);
    CHECK(u < v);
    CHECK(u < w);
    CHECK_FALSE(v < u);
    CHECK_FALSE(v < v);
    CHECK(v < w);
    CHECK_FALSE(w < u);
    CHECK_FALSE(w < v);
    CHECK_FALSE(w < w);

    CHECK(u <= u);
    CHECK(u <= v);
    CHECK(u <= w);
    CHECK_FALSE(v <= u);
    CHECK(v <= v);
    CHECK(v <= w);
    CHECK_FALSE(w <= u);
    CHECK_FALSE(w <= v);
    CHECK(w <= w);

    CHECK_FALSE(u > u);
    CHECK_FALSE(u > v);
    CHECK_FALSE(u > w);
    CHECK(v > u);
    CHECK_FALSE(v > v);
    CHECK_FALSE(v > w);
    CHECK(w > u);
    CHECK(w > v);
    CHECK_FALSE(w > w);

    CHECK(u >= u);
    CHECK_FALSE(u >= v);
    CHECK_FALSE(u >= w);
    CHECK(v >= u);
    CHECK(v >= v);
    CHECK_FALSE(v >= w);
    CHECK(w >= u);
    CHECK(w >= v);
    CHECK(w >= w);
}

TEST_CASE("Reverse_iterator") {
    String_vector v{"1", "2", "3"};

    auto i = v.rbegin();
    auto j = v.rend();

    CHECK(i < j);
    CHECK("3" == *i);
    CHECK("1" == i[2]);
    ++i;
    CHECK("2" == *i);
    CHECK("3" == i[-1]);
    ++i;
    CHECK("1" == *i);
    ++i;
    CHECK(i == j);
}
